/***************************************************************************
 *   Copyright (C) 2010 by Thomas Richard <thomas.richard@proan.be>        *
 *   Copyright (C) 2009-2010 by Peter Penz <peter.penz@gmx.at>             *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#ifndef DOLPHINBOXPLUGIN_H
#define DOLPHINBOXPLUGIN_H

#include <kversioncontrolplugin.h>
#include <QDateTime>
#include <QHash>
#include <QLocalSocket>
#include <QFileInfo>
#include <QTimer>

//The dropbox protocol info can be found in the dropboxd-protocol file
//it can be found here http://dl.dropbox.com/u/4577542/dropbox/dropboxd-protocol.txt
//Thanks Steffen Schuldenzucker!

/**
 * @brief Dropbox implementation for the KVersionControlPlugin interface.
 */
class DolphinBoxPlugin : public KVersionControlPlugin
{
    Q_OBJECT

public:
    DolphinBoxPlugin(QObject* parent, const QList<QVariant>& args);
    virtual ~DolphinBoxPlugin();
    virtual QString fileName() const;
    virtual bool beginRetrieval(const QString& directory);
    virtual void endRetrieval();
    virtual KVersionControlPlugin::VersionState versionState(const KFileItem& item);
    virtual QList<QAction*> contextMenuActions(const KFileItemList& items);
    virtual QList<QAction*> contextMenuActions(const QString& directory);

private slots:
    void browseAction();
    void revisionsAction();
    void showDeletedAction();
    void shareAction();
    void copyPublicURLAction();
    void copyGalleryURLAction();

    void checkIfUpdateNeeded();

private:
    QString sendCommand(QString string, QLocalSocket *socket, bool inGuiThread = true);
    QList<QAction*> getActions(QString url);

    bool m_pendingOperation;
    QHash<QString, VersionState> m_versionInfoHash;

    QAction* m_browseAction;
    QAction* m_revisionsAction;
    QAction* m_showDeletedAction;
    QAction* m_shareAction;
    QAction* m_copyPublicURLAction;
    QAction* m_copyGalleryURLAction;

    QString m_contextUrl;
    QFileInfo m_dbfile;
    QDateTime m_lastUpdate;
    QLocalSocket m_socket;
    QString m_socketpath;
    QTimer* m_timer;
};
#endif // DOLPHINBOXPLUGIN_H

